"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  default: () => babelPlugin
});
module.exports = __toCommonJS(src_exports);

// src/add-react-import.ts
function addReactImport(programPath) {
  let reactDefaultImportName = "React";
  let reactImport;
  programPath.traverse({
    ImportDeclaration(path) {
      if (path.node.source.value === "react") {
        reactImport = path.node;
        path.stop();
      }
    }
  });
  if (reactImport) {
    const defaultSpecifier = reactImport.specifiers.find(
      (specifier) => specifier.type === "ImportDefaultSpecifier"
    );
    if (!defaultSpecifier) {
      reactImport.specifiers.unshift({
        type: "ImportDefaultSpecifier",
        local: {
          type: "Identifier",
          name: "React"
        }
      });
    } else if (defaultSpecifier.local.name !== "React") {
      reactDefaultImportName = defaultSpecifier.local.name;
    }
  } else {
    programPath.node.body.unshift({
      type: "ImportDeclaration",
      specifiers: [
        {
          type: "ImportDefaultSpecifier",
          local: {
            type: "Identifier",
            name: "React"
          }
        }
      ],
      source: {
        type: "StringLiteral",
        value: "react"
      }
    });
  }
  return {
    reactDefaultImportName
  };
}

// src/transform-states.ts
var buildSetStateIdentifierName = (stateName) => `$$set_${stateName}`;
var buildStateVariableDeclaration = (stateName, init, opts) => ({
  type: "VariableDeclaration",
  kind: "const",
  declarations: [
    {
      type: "VariableDeclarator",
      id: {
        type: "ArrayPattern",
        elements: [
          {
            type: "Identifier",
            name: stateName
          },
          {
            type: "Identifier",
            name: buildSetStateIdentifierName(stateName)
          }
        ]
      },
      init: {
        type: "CallExpression",
        callee: {
          type: "MemberExpression",
          object: {
            type: "Identifier",
            name: opts.reactDefaultImportName
          },
          property: {
            type: "Identifier",
            name: "useState"
          },
          computed: false
        },
        arguments: [init]
      }
    }
  ]
});
var buildSetStateCallExpression = (stateName, args) => ({
  type: "CallExpression",
  callee: {
    type: "Identifier",
    name: buildSetStateIdentifierName(stateName)
  },
  arguments: args
});
var buildSetStateCallExpressionWithPrev = (stateName, body) => buildSetStateCallExpression(stateName, [
  {
    type: "ArrowFunctionExpression",
    expression: true,
    async: false,
    params: [
      {
        type: "Identifier",
        name: stateName
      }
    ],
    body
  }
]);
var assignmentOperators = [
  "+=",
  "-=",
  "*=",
  "/=",
  "%=",
  "**=",
  "<<=",
  ">>=",
  ">>>=",
  "&=",
  "^=",
  "|=",
  "&&=",
  "||=",
  "??="
];
function transformStates(path, opts) {
  const stateDeclarations = [];
  path.traverse({
    VariableDeclaration(path2) {
      const declaratorWithInit = path2.node.declarations[path2.node.declarations.length - 1];
      if (declaratorWithInit.id.type === "Identifier") {
        if (declaratorWithInit.init && declaratorWithInit.init.type === "CallExpression") {
          const callee = declaratorWithInit.init.callee;
          if (callee.type === "MemberExpression" && callee.object.type === "Identifier" && callee.object.name === "giga" && callee.property.type === "Identifier" && callee.property.name === "useState") {
            if (path2.node.kind !== "let") {
              throw new Error("giga.useState must be declared with let");
            }
            if (path2.node.declarations.length > 1) {
              throw new Error(
                "Only one state declarator is allowed per invocation of giga.useState"
              );
            }
            if (declaratorWithInit.init.arguments.length > 1) {
              throw new Error("giga.useState only accepts one argument");
            }
            stateDeclarations.push({
              path: path2,
              stateName: declaratorWithInit.id.name,
              init: declaratorWithInit.init.arguments[0]
            });
          }
        }
      }
    }
  });
  const stateNames = stateDeclarations.map((d) => d.stateName);
  stateDeclarations.forEach(({ path: path2, stateName, init }) => {
    const scopePath = path2.scope.path;
    path2.replaceWith(
      buildStateVariableDeclaration(stateName, init, {
        reactDefaultImportName: opts.reactDefaultImportName
      })
    );
    const stateMutations = [];
    scopePath.traverse({
      AssignmentExpression(path3) {
        if (path3.node.left.type === "Identifier") {
          if (stateNames.includes(path3.node.left.name)) {
            stateMutations.push({
              path: path3,
              stateName: path3.node.left.name
            });
          }
        }
      },
      UpdateExpression(path3) {
        if (path3.node.argument.type === "Identifier" && stateNames.includes(path3.node.argument.name)) {
          stateMutations.push({
            path: path3,
            stateName: path3.node.argument.name
          });
        }
      }
    });
    stateMutations.forEach(({ path: path3, stateName: stateName2 }) => {
      let newNode;
      if (path3.node.type === "UpdateExpression") {
        newNode = buildSetStateCallExpressionWithPrev(stateName2, {
          type: "BinaryExpression",
          left: {
            type: "Identifier",
            name: stateName2
          },
          operator: path3.node.operator === "++" ? "+" : "-",
          right: {
            type: "NumericLiteral",
            value: 1
          }
        });
      } else if (path3.node.type === "AssignmentExpression" && path3.node.left.type === "Identifier") {
        if (assignmentOperators.includes(path3.node.operator)) {
          newNode = buildSetStateCallExpressionWithPrev(stateName2, path3.node);
        } else {
          const rightPath = path3.get("right");
          let hasStateIdentifierInRight = false;
          rightPath.traverse({
            Identifier(path4) {
              if (path4.node.name === stateName2) {
                hasStateIdentifierInRight = true;
                path4.stop();
              }
            }
          });
          if (hasStateIdentifierInRight) {
            newNode = buildSetStateCallExpressionWithPrev(stateName2, path3.node.right);
          } else {
            newNode = buildSetStateCallExpression(stateName2, [path3.node]);
          }
        }
      }
      path3.replaceWith(newNode);
    });
  });
}

// src/index.ts
function babelPlugin() {
  return {
    name: "giga",
    visitor: {
      Program(programPath) {
        const { reactDefaultImportName } = addReactImport(programPath);
        const functionBodies = [];
        programPath.traverse({
          FunctionDeclaration(path) {
            functionBodies.push(path.get("body"));
          },
          FunctionExpression(path) {
            functionBodies.push(path.get("body"));
          },
          ArrowFunctionExpression(path) {
            functionBodies.push(path.get("body"));
          }
        });
        functionBodies.forEach((bodyPath) => {
          transformStates(bodyPath, {
            reactDefaultImportName
          });
        });
      }
    }
  };
}
